//****************************************************************************//
//                                                                            //
//                                TXYPlot                                     //
//               A Borland C++ Builder Plotting Component                     //
//               ----------------------------------------                     //
//               TXYPlot is copyright 1998 Jeff Nicholson                     //
//                             jeffn@rt66.com                                 //
//                                                                            //
//   This software is provided "as is" without express or implied warranty.   //
//****************************************************************************//
#include <vcl\vcl.h>
#pragma hdrstop

#include "xyplot.h"
#include <string.h>
#include <float.h>
#include <math.h>
//------------------------------------------------------------------------//
//************************************************************************//
//                              TPaintPage                                //
//************************************************************************//
//------------------------------------------------------------------------//
void TPaintPage::Initialize(void)
{
   PaintPage = new Graphics::TBitmap;
   PositionRect = new TRect;
   PageRect = new TRect;
}
//------------------------------------------------------------------------
void TPaintPage::CleanUp(void)
{
   delete PaintPage;
   delete PositionRect;
   delete PageRect;
}
//------------------------------------------------------------------------
TPaintPage::TPaintPage(int left, int top, int right, int bottom)
{
   PaintPage = new Graphics::TBitmap;

   PaintPage->Width = right-left;
   PaintPage->Height = top - bottom;
   PageRect->Left = 0;              // set PageRect to the size if the PaintPage
   PageRect->Right = right-left;
   PageRect->Top = 0;
   PageRect->Bottom = top-bottom;
   PositionRect->Left = left;       // set PositionRect to the position of the PaintPage
   PositionRect->Right = right;     // within the plot
   PositionRect->Top = top;
   PositionRect->Bottom = bottom;
}
//------------------------------------------------------------------------
void TPaintPage::SetPage(int left, int top, int right, int bottom)
{
   PaintPage->Width = right-left;
   PaintPage->Height = bottom-top;
   PageRect->Left = 0;
   PageRect->Right = right-left;
   PageRect->Top = 0;
   PageRect->Bottom = bottom-top;
   PositionRect->Left = left;
   PositionRect->Right = right;
   PositionRect->Top = top;
   PositionRect->Bottom = bottom;
}
//------------------------------------------------------------------------
void TPaintPage::SetPageWidth(int left, int right)
{
   PaintPage->Width = right-left;
   PageRect->Left = 0;
   PageRect->Right = right-left;
   PositionRect->Left = left;
   PositionRect->Right = right;
}
//------------------------------------------------------------------------
void TPaintPage::SetPageHeight(int top, int bottom)
{
   PaintPage->Height = bottom-top;
   PageRect->Top = 0;
   PageRect->Bottom = bottom-top;
   PositionRect->Top = top;
   PositionRect->Bottom = bottom;
}
//------------------------------------------------------------------------
void TPaintPage::ClearPaintPage(TColor Color)
{  // clears the paint page.
   PaintPage->Canvas->Brush->Style = bsSolid;
   PaintPage->Canvas->Brush->Color = Color;
   PaintPage->Canvas->FillRect(*PageRect);
}
//------------------------------------------------------------------------
void TPaintPage::AssignFont(TFont* Font)
{
   PaintPage->Canvas->Font->Assign(Font);
}
//------------------------------------------------------------------------//
//************************************************************************//
//                           TPlotListMember                              //
//************************************************************************//
//------------------------------------------------------------------------//
__fastcall TPlotListMember::TPlotListMember(const TPlotListMember* PlotListMember)
{   // copy constructor
   int i;

   Initialize();        //NextPtr is left NULL;
   NumPoints = PlotListMember->NumPoints;
   x = new double [NumPoints];
   y = new double [NumPoints];
   for(i=0;i<NumPoints;i++)
   {
      x[i] = PlotListMember->x[i];
      y[i] = PlotListMember->y[i];
   }
   Color = PlotListMember->Color;
   PlotStyle = PlotListMember->PlotStyle;
   Index = PlotListMember->Index;
}
//------------------------------------------------------------------------
void TPlotListMember::CleanUp(void)
{
   delete[] x;
   delete[] y;
   x = NULL;
   y = NULL;
}
//------------------------------------------------------------------------
void TPlotListMember::Initialize(void)
{
   x = NULL;
   y = NULL;
   NextPtr = NULL;
}
//------------------------------------------------------------------------//
//************************************************************************//
//                               TPlotList                                //
//************************************************************************//
//------------------------------------------------------------------------//
void TPlotList::CleanUp(void)
{
   TPlotListMember *CurrentPtr, *NxtPtr;

   CurrentPtr = FirstPtr;
   NxtPtr = FirstPtr;
   while(CurrentPtr != NULL)
   {
      CurrentPtr = CurrentPtr->NextPtr;
      delete NxtPtr;
      NxtPtr = CurrentPtr;
   }
   FirstPtr = NULL;
}
//------------------------------------------------------------------------
void TPlotList::Initialize(void)
{
   FirstPtr = NULL;
}
//------------------------------------------------------------------------
__fastcall TPlotList::TPlotList(const TPlotList* PlotList)
{   //copy constructor
   TPlotListMember *CurrentPtr, *NewPtr;

   FirstPtr = NULL;
   CurrentPtr = PlotList->FirstPtr;
   while(CurrentPtr !=NULL)
   {
      NewPtr = new TPlotListMember(CurrentPtr);
      NewPtr->NextPtr = FirstPtr;
      FirstPtr = NewPtr;
      CurrentPtr = CurrentPtr->NextPtr;
   }
}
//------------------------------------------------------------------------
void TPlotList::AppendPlotList(const TPlotList* PlotList)
{  // adds the data from PlotList to the current PlotList
   TPlotListMember *CurrentPtr, *NewPtr;

   CurrentPtr = PlotList->FirstPtr;
   while(CurrentPtr !=NULL)
   {
      AddPlot(CurrentPtr->x, CurrentPtr->y, CurrentPtr->NumPoints,
              CurrentPtr->Index, CurrentPtr->Color, CurrentPtr->PlotStyle);
      CurrentPtr = CurrentPtr->NextPtr;
   }
}
//------------------------------------------------------------------------
bool TPlotList::AddPlot(double xin_array[], double yin_array[],int NumPoints,
                        int PNumber, TColor PColor, TPlotStyle PStyle)
{  // adds a plot to the plot list.  If the plot with index PNumber already
//exists, deletes the old data, and replaces it with the new.
   int i;
   bool PlotFound;
   TPlotListMember* CurrentPtr;
   TPlotListMember* NewPtr;

   PlotFound = FindPlot(CurrentPtr, PNumber);
   if(!PlotFound)              // no plot with index PNumber exists, so create a new one
   {
      NewPtr = new TPlotListMember;
      NewPtr->y = new double[NumPoints];
      NewPtr->x = new double[NumPoints];
      for(i=0;i<NumPoints;i++)
      {
         NewPtr->y[i] = yin_array[i];
         NewPtr->x[i] = xin_array[i];
      }
      NewPtr->NumPoints = NumPoints;
      NewPtr->Index = PNumber;
      NewPtr->Color = PColor;
      NewPtr->PlotStyle = PStyle;
      NewPtr->NextPtr = FirstPtr;    // set the next ptr of the new member to the current first member
      FirstPtr = NewPtr;             // set the ptr to the first member to the new member
   }
   else                                        // Index already exists, so only modify old data
   {
      delete[] CurrentPtr->y;
      delete[] CurrentPtr->x;
      CurrentPtr->y = new double[NumPoints];
      CurrentPtr->x = new double[NumPoints];
      CurrentPtr->NumPoints = NumPoints;
      for(i=0;i<NumPoints;i++)
      {
         CurrentPtr->y[i] = yin_array[i];
         CurrentPtr->x[i] = xin_array[i];
      }
   }
   return(PlotFound);
}
//---------------------------------------------------------------------------
bool TPlotList::RemovePlot(int PNumber)
// deletes the plot with index PNumber.  If the plot exists, it is deleted, and
// a one is returned.  If a plot with index PNumber does not exist, a zero is returned
{
   TPlotListMember* CurrentPtr;
   TPlotListMember* OldPtr;
   TPlotListMember* DeletePtr;
   bool PlotFound = false;
//   TRect PlotPoint;

   CurrentPtr = FirstPtr;            // set current ptr to the first plot in the list
   while(CurrentPtr != NULL)
   {
      if(PNumber == CurrentPtr->Index)
      {
         PlotFound = true;
         if(CurrentPtr == FirstPtr)
         {
            DeletePtr = FirstPtr;
            FirstPtr = DeletePtr->NextPtr;
         }
         else
         {
            OldPtr->NextPtr = CurrentPtr->NextPtr;  // set the member before the one to delete to
            DeletePtr = CurrentPtr;                 // point to the one after the one to be deleted
         }
         delete DeletePtr;
         break;
      }
      OldPtr = CurrentPtr;                     //ptr to the member BEFORE the current one
      CurrentPtr = CurrentPtr->NextPtr;
   }

   return(PlotFound);
}
//---------------------------------------------------------------------------
bool TPlotList::FindPlot(TPlotListMember* &PlotPtr, int PNumber)
{ // sets PlotPtr to point to the plot with index PNumber
   bool PlotFound = false;

   PlotPtr = FirstPtr;
   while(PlotPtr !=NULL)
   {
      if(PlotPtr->Index == PNumber)
      {
         PlotFound = true;
         break;
      }
      PlotPtr = PlotPtr->NextPtr;
   }
   return PlotFound;
}
//------------------------------------------------------------------------//
//************************************************************************//
//                               TXYPlot                                  //
//************************************************************************//
//------------------------------------------------------------------------//
static inline TXYPlot *ValidCtrCheck()
{
	return new TXYPlot(NULL);
}
//---------------------------------------------------------------------------
int TXYPlot::matherr (struct exception *a)
{
  if (a->type == OVERFLOW)
  {
     a->retval = 1e300;
     return 1;
  }
  if (a->type == UNDERFLOW)
  {
     a->retval = 1e-300;
     return 1;
  }
  if (a->type == SING)
  {
     a->retval = 1e200;
     return 1;
  }
  return 0;
}
//---------------------------------------------------------------------------
__fastcall TXYPlot::TXYPlot(TComponent* Owner)
	: TPaintBox(Owner)
{// sets default values for variables to match default settings of published
// properties.
   SetProperties();
   Initialize();
}
//---------------------------------------------------------------------------
namespace Xyplot
{
	void __fastcall Register()
	{
		TComponentClass classes[1] = {__classid(TXYPlot)};
		RegisterComponents("Additional", classes, 0);
	}
}
//---------------------------------------------------------------------------
__fastcall TXYPlot::TXYPlot(TXYPlot& XYPlot): TPaintBox(XYPlot.Owner)
{
   AssignPlot(&XYPlot);
}
//---------------------------------------------------------------------------
void TXYPlot::AssignPlot(const TXYPlot* XYPlot)
{  // assigns the data and copies the property values of XYPlot
   CleanUp();
   Initialize();
   delete PlotList;
   PlotList = new TPlotList(XYPlot->PlotList);
   AssignPlotProperties(XYPlot);
   FindXMaxMin();
   FindYMaxMin();
   ResizePaintPage();
   if(FAutoRedraw)
      Redraw();
}
//---------------------------------------------------------------------------
void TXYPlot::AssignPlotData(const TXYPlot* XYPlot)
{  // assigns the data of XYPlot (copies the plotlist)
   delete PlotList;
   PlotList = new TPlotList(XYPlot->PlotList);
   FindXMaxMin();
   FindYMaxMin();
   ResizePaintPage();
   if(FAutoRedraw)
      Redraw();
}
//---------------------------------------------------------------------------
void TXYPlot::AppendPlotData(const TXYPlot* XYPlot)
{  // appends the plotlist of XYPlot to the this PlotList.
   PlotList->AppendPlotList(XYPlot->PlotList);
   FindXMaxMin();
   FindYMaxMin();
   ResizePaintPage();
   if(FAutoRedraw)
      Redraw();
}
//---------------------------------------------------------------------------
void TXYPlot::AssignPlotProperties(const TXYPlot* XYPlot)
{  // copies the property values (but not the plotlist) of XYPlot
   FAutoRedraw=XYPlot->FAutoRedraw;
   FAxisColor=XYPlot->FAxisColor;
   FAxisStyle=XYPlot->FAxisStyle;
   FXAxisFormat=XYPlot->FXAxisFormat;
   FYAxisFormat=XYPlot->FYAxisFormat;
   FShowXLabel=XYPlot->FShowXLabel;
   FShowYLabel =XYPlot->FShowYLabel;
   FShowCoordinates=XYPlot->FShowCoordinates;
   FYAutoScale =XYPlot->FYAutoScale;
   FXAutoScale =XYPlot->FXAutoScale;
   FYMax =XYPlot->FYMax;
   FYMin =XYPlot->FYMin;
   FXMax =XYPlot->FXMax;
   FXMin =XYPlot->FXMin;
   FXLabelPrecision =XYPlot->FXLabelPrecision;
   FYLabelPrecision =XYPlot->FYLabelPrecision;
   FXLabelDigits =XYPlot->FXLabelDigits;
   FYLabelDigits =XYPlot->FYLabelDigits;
   FYLabelFormat =XYPlot->FYLabelFormat;
   FXLabelFormat=XYPlot->FXLabelFormat;
   FPointSize=XYPlot->FPointSize;
   FShowHorzGrid=XYPlot->FShowHorzGrid;
   FShowVertGrid=XYPlot->FShowVertGrid;
   FShowHorzTicks=XYPlot->FShowHorzTicks;
   FShowVertTicks=XYPlot->FShowVertTicks;
   FVertTickLength=XYPlot->FVertTickLength;
   FHorzTickLength=XYPlot->FHorzTickLength;
   FNumHorzGrid=XYPlot->FNumHorzGrid;
   FNumVertGrid=XYPlot->FNumVertGrid;
   FGridColor=XYPlot->FGridColor;
   FMouseControl=XYPlot->FMouseControl;
   FShowXTitle=XYPlot->FShowXTitle;
   FShowYTitle=XYPlot->FShowYTitle;
   FShowPlotTitle =XYPlot->FShowPlotTitle;
   FXTitle =XYPlot->FXTitle;
   FYTitle=XYPlot->FYTitle;
   FPlotTitle=XYPlot->FPlotTitle;
   FPlotTitleFont->Assign(XYPlot->FPlotTitleFont);
   FAxisTitleFont->Assign(XYPlot->FAxisTitleFont);
   FAxisLabelFont->Assign(XYPlot->FAxisLabelFont);
   FMouseCoordFont->Assign(XYPlot->FMouseCoordFont);
   if(FAutoRedraw)
      Redraw();
}
//---------------------------------------------------------------------------
void TXYPlot::SetProperties(void)
{  // sets default property values
   FAutoRedraw = true;
   FAxisColor = clBlack;
   FAxisStyle = CenteredBox;
   FXAxisFormat = Linear;
   FYAxisFormat = Linear;
   FShowXLabel = true;
   FShowYLabel = true;
   FShowCoordinates = true;
   FYAutoScale = true;
   FXAutoScale = true;
   FYMax = 1;
   FYMin = -1;
   FXMax = 1;
   FXMin = -1;
   FXLabelPrecision = 4;
   FYLabelPrecision = 4;
   FXLabelDigits = 4;
   FYLabelDigits = 4;
   FYLabelFormat = AnsiString::sffGeneral;
   FXLabelFormat = AnsiString::sffGeneral;
   FPointSize = 2;
   FShowHorzGrid = true;
   FShowVertGrid = true;
   FShowHorzTicks = true;
   FShowVertTicks = true;
   FVertTickLength = 6;
   FHorzTickLength = 6;
   FNumHorzGrid = 3;
   FNumVertGrid = 3;
   FGridColor = TColor(0x00D8E09E);
   FMouseControl = true;
   FShowXTitle = true;
   FShowYTitle = true;
   FShowPlotTitle = true;
   FXTitle = "X Axis";
   FYTitle = "Y Axis";
   FPlotTitle = "XY Plot";
   DragPlot = false;
   ZoomPlot = false;
   BoxZoomPlot = false;
   Height = 250;
   Width = 300;
   OldWidth = 0;
   OldHeight = 0;
   FMouseX = 0;
   FMouseY = 0;
}
//---------------------------------------------------------------------------
void TXYPlot::Initialize(void)
{  // allocates memory
   PlotList = new TPlotList;
   PlotPage = new TPaintPage;
   XLabelPage = new TPaintPage;
   YLabelPage = new TPaintPage;
   PlotTitlePage = new TPaintPage;
   XTitlePage = new TPaintPage;
   YTitlePage = new TPaintPage;
   YTempPage = new TPaintPage;
   MouseCoordPage = new TPaintPage;
   FPlotTitleFont = new TFont;
   FAxisTitleFont = new TFont;
   FAxisLabelFont = new TFont;
   FMouseCoordFont = new TFont;
}
//---------------------------------------------------------------------------
void TXYPlot::CleanUp(void)
{  // frees up memory
   delete PlotList;
   delete PlotPage;
   delete XLabelPage;
   delete YLabelPage;
   delete PlotTitlePage;
   delete XTitlePage;
   delete YTitlePage;
   delete YTempPage;
   delete MouseCoordPage;
   delete FPlotTitleFont;
   delete FAxisTitleFont;
   delete FAxisLabelFont;
   delete FMouseCoordFont;
}
//---------------------------------------------------------------------------
bool TXYPlot::XYPlot(double xin_array[], double yin_array[],
                              int NumPoints, int PNumber,
                              TColor PColor, TPlotStyle PStyle)
// plots x and y array.
{
   bool PlotFound;

   PlotFound = PlotList->AddPlot(xin_array, yin_array, NumPoints, PNumber, PColor, PStyle);

   FindXMaxMin();
   FindYMaxMin();
   if(FAutoRedraw)
      DrawPlotsAndLabels();
   return(PlotFound);
}
//---------------------------------------------------------------------------
bool TXYPlot::XYPlot(double in_array[], int NumPoints, int PNumber,
                                TColor PColor,TPlotStyle PStyle)
// Plots a y array only.  x array is left alone
{
   int i;
   bool PlotFound;
   double* x = new double[NumPoints];

   for(i=0;i<NumPoints;i++)
      x[i] = i;
   PlotFound = PlotList->AddPlot(x, in_array, NumPoints, PNumber, PColor, PStyle);

   FindXMaxMin();
   FindYMaxMin();
   if(FAutoRedraw)
      DrawPlotsAndLabels();
   delete[] x;
   return(PlotFound);
}
//---------------------------------------------------------------------------
bool TXYPlot::XYPlot(double in_array[], int NumPoints, double minX,
                               double maxX, int PNumber, TColor PColor, TPlotStyle PStyle)
// Plots a y array only.  x array is scaled between minX and maxX
{
   int i;
   double dx;
   bool PlotFound;
   double* x = new double[NumPoints];

   dx = (maxX-minX)/(NumPoints-1.0);
   for(i=0;i<NumPoints;i++)
      x[i] = minX + dx*i;
   PlotFound = PlotList->AddPlot(x, in_array, NumPoints, PNumber, PColor, PStyle);

   FindXMaxMin();
   FindYMaxMin();
   if(FAutoRedraw)
      DrawPlotsAndLabels();
   delete[] x;
   return(PlotFound);
}
//---------------------------------------------------------------------------
bool TXYPlot::RemovePlot(int PNumber)
{
   bool PlotFound;

   PlotFound = PlotList->RemovePlot(PNumber);
   if (PlotFound)
   {
      FindXMaxMin();
      FindYMaxMin();
      if(FAutoRedraw)
         DrawPlotsAndLabels();
   }
   return PlotFound;
}
//---------------------------------------------------------------------------
void TXYPlot::Redraw(void)
{
   DrawPlotsAndLabels();
   DrawTitles();
}
//---------------------------------------------------------------------------
void TXYPlot::DrawAxis(void)
{
   if(Visible)
   {
      // Draw the axis
      PlotPage->PaintPage->Canvas->Pen->Color = FAxisColor;
      PlotPage->PaintPage->Canvas->Brush->Style = bsSolid;
      PlotPage->PaintPage->Canvas->Brush->Color = FAxisColor;
      switch (FAxisStyle)
      {
      case Box         : PlotPage->PaintPage->Canvas->FrameRect(*(PlotPage->PageRect));
                         break;
      case Centered    : if(FYMax!=FYMin)
                         {
                            PlotPage->PaintPage->Canvas->MoveTo(0, ScreenY(0));
                            PlotPage->PaintPage->Canvas->LineTo(PlotPage->PaintPage->Width, ScreenY(0));
                         }
                         if(FXMax!=FXMin)
                         {
                            PlotPage->PaintPage->Canvas->MoveTo(ScreenX(0),0);
                            PlotPage->PaintPage->Canvas->LineTo(ScreenX(0),PlotPage->PaintPage->Height);
                         }
                         break;
      case CenteredBox : PlotPage->PaintPage->Canvas->FrameRect(*(PlotPage->PageRect));
                         if(FYMax!=FYMin)
                         {
                            PlotPage->PaintPage->Canvas->MoveTo(0, ScreenY(0));
                            PlotPage->PaintPage->Canvas->LineTo(PlotPage->PaintPage->Width, ScreenY(0));
                         }
                         if(FXMax!=FXMin)
                         {
                            PlotPage->PaintPage->Canvas->MoveTo(ScreenX(0),0);
                            PlotPage->PaintPage->Canvas->LineTo(ScreenX(0),PlotPage->PaintPage->Height);
                         }
                         break;
      case LB          : PlotPage->PaintPage->Canvas->MoveTo(0,0);
                         PlotPage->PaintPage->Canvas->LineTo(0,PlotPage->PaintPage->Height-1);
                         PlotPage->PaintPage->Canvas->LineTo(PlotPage->PaintPage->Width,PlotPage->PaintPage->Height-1);
                         break;
      case None        : break;
      }// end switch(FAxisStyle)
   }
}
//---------------------------------------------------------------------------
void TXYPlot::DrawXAxisLabels(void)
{  // draw the x axis labels at the same place as the vertical ticks and grid lines
   AnsiString St;
   double delta,xx;
   int i, exponent;
   int textleft;

   if(FShowXLabel)
   {
      XLabelPage->ClearPaintPage(Color);
      XLabelPage->PaintPage->Canvas->Font->Assign(FAxisLabelFont);
      if(FXAxisFormat == Linear)
      {
         St = St.FloatToStrF(double(FXMin),FXLabelFormat,FXLabelPrecision,FXLabelDigits);
         XLabelPage->PaintPage->Canvas->TextOut(1,1,St);
         St = St.FloatToStrF(double(FXMax),FXLabelFormat,FXLabelPrecision,FXLabelDigits);
         textleft = XLabelPage->PaintPage->Width - XLabelPage->PaintPage->Canvas->TextWidth(St);
         XLabelPage->PaintPage->Canvas->TextOut(textleft,1,St);
         delta = (FXMax-FXMin)/(FNumVertGrid+1);
         for(i=0;i<FNumVertGrid;i++)
         {
            xx = ScreenX(FXMin + delta*(i+1));
            St = St.FloatToStrF(FXMin + delta*(i+1),FXLabelFormat,FXLabelPrecision,FXLabelDigits);
            textleft = xx - double(XLabelPage->PaintPage->Canvas->TextWidth(St))/2;
            XLabelPage->PaintPage->Canvas->TextOut(textleft,1,St);
         }
      }
      else if (FXAxisFormat == Log)
      {
         exponent = int(floor(XLogMin))+1;
         while (exponent < XLogMax)
         {
            xx = ScreenX(pow(10,exponent));
            St = St.FloatToStrF(pow(10,exponent),FXLabelFormat,FXLabelPrecision,FXLabelDigits);
            textleft = xx - double(XLabelPage->PaintPage->Canvas->TextWidth(St))/2;
            XLabelPage->PaintPage->Canvas->TextOut(textleft,1,St);
            exponent +=1;
         }
      }
      UpdatePaintPage(XLabelPage);
   }
}
//---------------------------------------------------------------------------
void TXYPlot::DrawYAxisLabels(void)
{  // draw the y axis labels centered on the horizontal grid and tick lines
   AnsiString St;
   double delta,yy;
   int i, exponent;
   int textleft;
   int texttop;

   if(FShowYLabel)
   {
      YLabelPage->ClearPaintPage(Color);
      YLabelPage->PaintPage->Canvas->Font->Assign(FAxisLabelFont);
      if(FYAxisFormat == Linear)
      {
         St = St.FloatToStrF(double(FYMax),FYLabelFormat,FYLabelPrecision,FYLabelDigits);
         textleft = YLabelPage->PaintPage->Width - YLabelPage->PaintPage->Canvas->TextWidth(St) - 2;
         YLabelPage->PaintPage->Canvas->TextOut(textleft,0,St);
         St=St.FloatToStrF(double(FYMin),FYLabelFormat,FYLabelPrecision,FYLabelDigits);
         textleft = YLabelPage->PaintPage->Width - YLabelPage->PaintPage->Canvas->TextWidth(St) - 2;
         texttop = ScreenY(FYMin) - YLabelPage->PaintPage->Canvas->TextHeight(FYMin);
         YLabelPage->PaintPage->Canvas->TextOut(textleft,texttop,St);
         delta = (FYMax-FYMin)/(FNumHorzGrid+1);
         for(i=0;i<FNumHorzGrid;i++)
         {
            yy = ScreenY(FYMin + delta*(i+1));
            St =St.FloatToStrF(FYMin + delta*(i+1),FYLabelFormat,FYLabelPrecision,FYLabelDigits);
            textleft = YLabelPage->PaintPage->Width - YLabelPage->PaintPage->Canvas->TextWidth(St) - 2;
            texttop = YLabelPage->PaintPage->Canvas->TextHeight(St)/2;
            YLabelPage->PaintPage->Canvas->TextOut(textleft,yy-texttop,St);
         }
      }
      else if (FYAxisFormat == Log)
      {
         exponent = int(floor(YLogMin))+1;
         while (exponent < YLogMax)
         {
            yy = ScreenY(pow(10,exponent));
            St = St.FloatToStrF(pow(10,exponent),FYLabelFormat,FYLabelPrecision,FYLabelDigits);
            textleft = YLabelPage->PaintPage->Width - YLabelPage->PaintPage->Canvas->TextWidth(St) - 2;
            texttop = YLabelPage->PaintPage->Canvas->TextHeight(St)/2;
            YLabelPage->PaintPage->Canvas->TextOut(textleft,yy-texttop,St);
            exponent +=1;
         }
      }
      UpdatePaintPage(YLabelPage);
   }
}
//---------------------------------------------------------------------------
inline double TXYPlot::ScreenX(double X)
{  // given X, it returns the x coordinate in the plotpage
   double xx;

   switch (FXAxisFormat)
   {
      case Linear :
         xx = PlotPage->PaintPage->Width *(X-FXMin)/(FXMax-FXMin);
         break;
      case Log :
         if (X>0)
            X = log10(X);
         else
            X = log10(FXMin/10);
         xx = PlotPage->PaintPage->Width *(X-XLogMin)/(XLogMax-XLogMin);
         break;
   }

   return xx;
}
//---------------------------------------------------------------------------
inline double TXYPlot::ScreenY(double Y)
{  // Given Y ScreenY returns the vertical coordinate value in the plotpage
   double yy;

   switch (FYAxisFormat)
   {
      case Linear :
         yy = PlotPage->PaintPage->Height-PlotPage->PaintPage->Height*(Y-FYMin)/(FYMax-FYMin);
         break;
      case Log :
         if (Y>0)
            Y = log10(Y);
         else
            Y = log10(FYMin/10);
         yy = PlotPage->PaintPage->Height-PlotPage->PaintPage->Height*(Y-YLogMin)/(YLogMax-YLogMin);
         break;
   }
   return yy;
}
//---------------------------------------------------------------------------
inline double TXYPlot::PlotX(double X)
{
   double xx;

   if(FXAxisFormat == Linear)          // copy the mouse coordinates to MouseX and MouseY
      xx = double(X-PlotPage->PositionRect->Left)*(FXMax-FXMin)/PlotPage->PaintPage->Width + FXMin;
   else if (FXAxisFormat == Log)
      xx = pow(10,double(X-PlotPage->PositionRect->Left)*(XLogMax-XLogMin)/PlotPage->PaintPage->Width + XLogMin);
   return xx;
}
//---------------------------------------------------------------------------
inline double TXYPlot::PlotY(double Y)
{
   double yy;

   if (FYAxisFormat == Linear)
      yy = -(double(Y-PlotPage->PositionRect->Top)-PlotPage->PaintPage->Height)*(FYMax-FYMin)/PlotPage->PaintPage->Height+FYMin;
   else if (FYAxisFormat == Log)
      yy = pow(10,-(double(Y-PlotPage->PositionRect->Top)-PlotPage->PaintPage->Height)*(YLogMax-YLogMin)/PlotPage->PaintPage->Height+YLogMin);
   return yy;
}
//---------------------------------------------------------------------------
void TXYPlot::DrawOnePlot(TPlotListMember* PlotPtr)
{  // draws the plot, pointed to by PlotPtr.
   int i,xx,yy;

   if(Visible)
   {
       PlotPage->PaintPage->Canvas->Pen->Color = PlotPtr->Color;
      if ((PlotPtr->PlotStyle == FilledPoints)||
          (PlotPtr->PlotStyle == LinesFilledPoints))
         PlotPage->PaintPage->Canvas->Brush->Color = PlotPtr->Color;
      else
         PlotPage->PaintPage->Canvas->Brush->Color = Color;
         xx =ScreenX(PlotPtr->x[0]);
         yy = ScreenY(PlotPtr->y[0]);
         PlotPage->PaintPage->Canvas->MoveTo(xx,yy);
      for (i=0;i<PlotPtr->NumPoints;i++)
      {
         xx = ScreenX(PlotPtr->x[i]);
         yy = ScreenY(PlotPtr->y[i]);
         if ((xx>=-2*PlotPage->PaintPage->Width)&&(xx<(3*PlotPage->PaintPage->Width))&&(yy>=-2*PlotPage->PaintPage->Height)&&(yy<(3*PlotPage->PaintPage->Height)))
         {
            if(PlotPtr->PlotStyle == Lines)
               PlotPage->PaintPage->Canvas->LineTo(xx,yy);
            else if ((PlotPtr->PlotStyle == OpenPoints)||(PlotPtr->PlotStyle == FilledPoints))
               PlotPage->PaintPage->Canvas->Rectangle(xx-FPointSize,yy-FPointSize,xx+FPointSize,yy+FPointSize);
            else if ((PlotPtr->PlotStyle == LinesOpenPoints)||(PlotPtr->PlotStyle == LinesFilledPoints))
            {
               PlotPage->PaintPage->Canvas->LineTo(xx,yy);
               PlotPage->PaintPage->Canvas->Rectangle(xx-FPointSize,yy-FPointSize,xx+FPointSize,yy+FPointSize);
               PlotPage->PaintPage->Canvas->MoveTo(xx,yy);
            }
         }
         else
            PlotPage->PaintPage->Canvas->MoveTo(xx,yy);
      } // end for
   }
}
//---------------------------------------------------------------------------
void TXYPlot::DrawAllPlots(void)
{
   // Draw the Grid, ticks, axis, and plots
   TPlotListMember* CurrentPtr;

   if(Visible)
   {
      PlotPage->ClearPaintPage(Color);
      DrawGrid();
      DrawTicks();
      DrawAxis();
      CurrentPtr = PlotList->FirstPtr;
      while(CurrentPtr != NULL)
      {
         DrawOnePlot(CurrentPtr);
         CurrentPtr = CurrentPtr->NextPtr;
      }// end while(CurrentPtr != NULL)
      UpdatePaintPage(PlotPage);
   }
}
//---------------------------------------------------------------------------
void TXYPlot::DrawGrid(void)
{   // draws the grid onto the plotpage.
   int i,exponent;
   double xx,yy;
   double delta;
   PlotPage->PaintPage->Canvas->Pen->Color = FGridColor;
   if(FShowHorzGrid)
   {
      if (FYAxisFormat == Linear)
      {
         delta = (FYMax-FYMin)/(FNumHorzGrid+1);
         for(i=0;i<FNumHorzGrid;i++)
         {
            yy = ScreenY(FYMin + delta*(i+1));
            PlotPage->PaintPage->Canvas->MoveTo(0,yy);
            PlotPage->PaintPage->Canvas->LineTo(PlotPage->PaintPage->Width,yy);
         }
      }
      else if (FYAxisFormat == Log)
      {
         exponent = int(floor(YLogMin));
         while (exponent < YLogMax)
         {
            yy = ScreenY(pow(10,exponent));
            PlotPage->PaintPage->Canvas->MoveTo(0,yy);
            PlotPage->PaintPage->Canvas->LineTo(PlotPage->PaintPage->Width,yy);
            delta = (pow(10,exponent+1) - pow(10,exponent))/(FNumHorzGrid+1);
            for(i=0;i<FNumHorzGrid;i++)
            {
               yy = ScreenY(pow(10,exponent) + delta*(i+1));
               PlotPage->PaintPage->Canvas->MoveTo(0,yy);
               PlotPage->PaintPage->Canvas->LineTo(PlotPage->PaintPage->Width,yy);
             }
            exponent +=1;
         }
      }
   }
   if(FShowVertGrid)
   {
      if (FXAxisFormat==Linear)
      {
         delta = (FXMax-FXMin)/(FNumVertGrid+1);
         for(i=0;i<FNumVertGrid;i++)
         {
            xx = ScreenX(FXMin + delta*(i+1));
            PlotPage->PaintPage->Canvas->MoveTo(xx,0);
            PlotPage->PaintPage->Canvas->LineTo(xx,PlotPage->PaintPage->Height);
         }
      }
      else if (FXAxisFormat == Log)
      {
         exponent = int(floor(XLogMin));
         while (exponent < XLogMax)
         {
            xx = ScreenX(pow(10,exponent));
            PlotPage->PaintPage->Canvas->MoveTo(xx,0);
            PlotPage->PaintPage->Canvas->LineTo(xx,PlotPage->PaintPage->Height);
            delta = (pow(10,exponent+1) - pow(10,exponent))/(FNumVertGrid+1);
            for(i=0;i<FNumVertGrid;i++)
            {
               xx = ScreenX(pow(10,exponent) + delta*(i+1));
               PlotPage->PaintPage->Canvas->MoveTo(xx,0);
               PlotPage->PaintPage->Canvas->LineTo(xx,PlotPage->PaintPage->Height);
            }
            exponent +=1;
         }
      }
   }
}
//---------------------------------------------------------------------------
void TXYPlot::DrawTicks(void)
{  // draws the ticks, onto the plotpage.  Puts them in the same place as the grid
   int i,exponent;
   double xx,yy;
   double delta;
   PlotPage->PaintPage->Canvas->Pen->Color = FAxisColor;
   if(FShowHorzTicks)
   {
      if (FYAxisFormat == Linear)
      {
         delta = (FYMax-FYMin)/(FNumHorzGrid+1);
         for(i=0;i<FNumHorzGrid;i++)
         {
            yy = ScreenY(FYMin + delta*(i+1));
            PlotPage->PaintPage->Canvas->MoveTo(0,yy);
            PlotPage->PaintPage->Canvas->LineTo(FHorzTickLength,yy);
         }
      }
      else if (FYAxisFormat == Log)
      {
         exponent = int(floor(YLogMin));
         while (exponent < YLogMax)
         {
            yy = ScreenY(pow(10,exponent));
            PlotPage->PaintPage->Canvas->MoveTo(0,yy);
            PlotPage->PaintPage->Canvas->LineTo(FHorzTickLength,yy);
            delta = (pow(10,exponent+1) - pow(10,exponent))/(FNumHorzGrid+1);
            for(i=0;i<FNumHorzGrid;i++)
            {
               yy = ScreenY(pow(10,exponent) + delta*(i+1));
               PlotPage->PaintPage->Canvas->MoveTo(0,yy);
               PlotPage->PaintPage->Canvas->LineTo(FHorzTickLength,yy);
             }
            exponent +=1;
         }
      }
   }
   if(FShowVertTicks)
   {
      if (FXAxisFormat==Linear)
      {
         delta = (FXMax-FXMin)/(FNumVertGrid+1);
         for(i=0;i<FNumVertGrid;i++)
         {
            xx = ScreenX(FXMin + delta*(i+1));
            PlotPage->PaintPage->Canvas->MoveTo(xx,PlotPage->PaintPage->Height);
            PlotPage->PaintPage->Canvas->LineTo(xx,PlotPage->PaintPage->Height-FVertTickLength);
         }
      }
      else if (FXAxisFormat == Log)
      {
         exponent = int(floor(XLogMin));
         while (exponent < XLogMax)
         {
            xx = ScreenX(pow(10,exponent));
            PlotPage->PaintPage->Canvas->MoveTo(xx,PlotPage->PaintPage->Height);
            PlotPage->PaintPage->Canvas->LineTo(xx,PlotPage->PaintPage->Height-FVertTickLength);
            delta = (pow(10,exponent+1) - pow(10,exponent))/(FNumVertGrid+1);
            for(i=0;i<FNumVertGrid;i++)
            {
               xx = ScreenX(pow(10,exponent) + delta*(i+1));
               PlotPage->PaintPage->Canvas->MoveTo(xx,PlotPage->PaintPage->Height);
               PlotPage->PaintPage->Canvas->LineTo(xx,PlotPage->PaintPage->Height-FVertTickLength);
            }
            exponent +=1;
         }
      }
   }
}
//---------------------------------------------------------------------------
void TXYPlot::ClearPlot(void)
{  // clears all the paintpages.
   if(FShowXLabel)
      XLabelPage->ClearPaintPage(Color);
   if(FShowXTitle)
      XTitlePage->ClearPaintPage(Color);
   if(FShowYLabel)
      YLabelPage->ClearPaintPage(Color);
   if(FShowYTitle)
   {
      YTitlePage->ClearPaintPage(Color);
      YTempPage->ClearPaintPage(Color);
   }
   if(FShowPlotTitle)
      PlotTitlePage->ClearPaintPage(Color);
   PlotPage->ClearPaintPage(Color);
}
//---------------------------------------------------------------------------
inline void TXYPlot::UpdatePaintPage(TPaintPage* Page)
{  //copies the canvas of the PaintPage Page onto the main Canvas
   Canvas->CopyRect(*(Page->PositionRect),
                    Page->PaintPage->Canvas,*(Page->PageRect));
}
//---------------------------------------------------------------------------
inline void TXYPlot::RedrawScreen(void)
{  // Copies all the paintpages
   if(FShowXLabel)
      UpdatePaintPage(XLabelPage);
   if(FShowXTitle)
      UpdatePaintPage(XTitlePage);
   if(FShowYLabel)
      UpdatePaintPage(YLabelPage);
   if(FShowYTitle)
      UpdatePaintPage(YTitlePage);
   if(FShowPlotTitle)
      UpdatePaintPage(PlotTitlePage);
   UpdatePaintPage(PlotPage);
}
//---------------------------------------------------------------------------
void TXYPlot::ResizePaintPage(void)
{  // sizes the paintpages, depending on which labels and titles are visible.
// if the width of the plot is negative, Y axis titles and labels are
// turned off, and resize is called again.  If the Height of the plotpage is
// negative, the PlotTitle, and X axis titles and labels are turned off,
// and resize is called again.
   int plotleft = 0;
   int labelleft = 0;
   int plotbottom = Height;
   int labelbottom = Height;
   int top = 0;
   int l, exponent;
   int templeft=0;
   int tempbottom=0;
   int precision;
   int digits;
   int textbottom;
   AnsiString St;

   if (FShowXLabel)
   {
      XLabelPage->AssignFont(FAxisLabelFont);
      St = St.FloatToStrF(FXMin,FXLabelFormat,FXLabelPrecision,FXLabelDigits);
      plotbottom -= 1.2*XLabelPage->PaintPage->Canvas->TextHeight(St);
   }
   if(FShowXTitle)
   {
      XTitlePage->AssignFont(FAxisTitleFont);
      plotbottom -= 2*XTitlePage->PaintPage->Canvas->TextHeight(FXTitle);
      labelbottom -= 2*XTitlePage->PaintPage->Canvas->TextHeight(FXTitle);
   }
   if (FShowYLabel)
   {
      YLabelPage->AssignFont(FAxisLabelFont);
      l = 0;
      for (exponent = -100; exponent <= 100; exponent++)
      {
         precision = (FXLabelPrecision > FYLabelPrecision) ? FXLabelPrecision : FYLabelPrecision;
         digits = (FXLabelDigits > FYLabelDigits) ? FXLabelDigits : FYLabelDigits;
         St = St.FloatToStrF(M_PI*pow(10,exponent),FXLabelFormat,precision,digits);
         if (YLabelPage->PaintPage->Canvas->TextWidth(St) > l)
            l = YLabelPage->PaintPage->Canvas->TextWidth(St);
      }
      plotleft += l+4;
   }
   if (FShowYTitle)
   {
      YTempPage->AssignFont(FAxisTitleFont);
      YTitlePage->AssignFont(FAxisTitleFont);
      plotleft += 2*YTitlePage->PaintPage->Canvas->TextHeight(FYTitle);
      labelleft += 2*YTitlePage->PaintPage->Canvas->TextHeight(FYTitle);
      templeft = YTitlePage->PaintPage->Canvas->TextWidth(FYTitle);
      tempbottom = YTitlePage->PaintPage->Canvas->TextHeight(FYTitle);
   }
   if (FShowPlotTitle)
   {
      PlotTitlePage->AssignFont(FPlotTitleFont);
      top += 2*PlotTitlePage->PaintPage->Canvas->TextHeight(FPlotTitle);
   }
   if (FShowCoordinates)
   {
      MouseCoordPage->AssignFont(FMouseCoordFont);
      l = 0;
      for (exponent = -100; exponent <= 100; exponent++)
      {
         St = "x = " + St.FloatToStrF(M_PI*pow(10,exponent),FYLabelFormat,FYLabelPrecision,FYLabelDigits);
         if (MouseCoordPage->PaintPage->Canvas->TextWidth(St) > l)
            l = MouseCoordPage->PaintPage->Canvas->TextWidth(St);

      }
      textbottom = MouseCoordPage->PaintPage->Canvas->TextHeight(St)*2+10;
      MouseCoordPage->SetPage(Width-l-5,top+4,Width-5,top+2+textbottom);
   }
   if ((top < plotbottom)&&(plotleft < Width))
   {
      PlotPage->SetPage(plotleft,top,Width,plotbottom);
      XLabelPage->SetPage(plotleft,plotbottom,Width,labelbottom);
      XTitlePage->SetPage(plotleft,labelbottom,Width,Height);
      YLabelPage->SetPage(labelleft,top,plotleft,Height);
      YTitlePage->SetPage(0,top,labelleft,Height);
      YTempPage->SetPage(0,0,templeft,tempbottom);
      PlotTitlePage->SetPage(0,0,Width,top);
   }
   if(top > plotbottom)
   {
      FShowPlotTitle = false;
      FShowXLabel = false;
      FShowXTitle = false;
      ResizePaintPage();
   }
   if(plotleft > Width)
   {
      FShowYLabel = false;
      FShowYTitle = false;
      ResizePaintPage();
   }
}
//---------------------------------------------------------------------------
void TXYPlot::DrawTitles(void)
{  // draws the plot title, and axis titles.
   int textleft;
   int texttop;
   int i,j;
   if(FShowPlotTitle)
   {
      PlotTitlePage->ClearPaintPage(Color);
      PlotTitlePage->PaintPage->Canvas->Font->Assign(FPlotTitleFont);
      texttop = PlotTitlePage->PaintPage->Height -
                 3*PlotTitlePage->PaintPage->Canvas->TextHeight(FPlotTitle)/2;
      textleft = PlotPage->PositionRect->Left + (PlotPage->PaintPage->Width -
                  PlotTitlePage->PaintPage->Canvas->TextWidth(FPlotTitle))/2;
      PlotTitlePage->PaintPage->Canvas->TextOut(textleft,texttop,FPlotTitle);
      UpdatePaintPage(PlotTitlePage);
   }
   if(FShowXTitle)
   {
      XTitlePage->ClearPaintPage(Color);
      XTitlePage->PaintPage->Canvas->Font->Assign(FAxisTitleFont);
      texttop = XTitlePage->PaintPage->Height -
                3*XTitlePage->PaintPage->Canvas->TextHeight(FXTitle)/2;
      textleft = (XTitlePage->PaintPage->Width -
                  XTitlePage->PaintPage->Canvas->TextWidth(FXTitle))/2;
      XTitlePage->PaintPage->Canvas->TextOut(textleft, texttop, FXTitle);
      UpdatePaintPage(XTitlePage);
   }
   if (FShowYTitle)
   {
      YTitlePage->ClearPaintPage(Color);
      YTempPage->ClearPaintPage(Color);
      YTitlePage->PaintPage->Canvas->Font->Assign(FAxisTitleFont);
      YTempPage->PaintPage->Canvas->Font->Assign(FAxisTitleFont);
      texttop = (PlotPage->PaintPage->Height -
                YTitlePage->PaintPage->Canvas->TextWidth(FYTitle))/2;
      textleft = YTitlePage->PaintPage->Canvas->TextHeight(FYTitle)/2;
      YTempPage->PaintPage->Canvas->TextOut(0,0,FYTitle);
      for(i=0;i<YTempPage->PaintPage->Width;i++)             // rotate the y title 90 deg.
         for(j=0;j<YTempPage->PaintPage->Height;j++)
            YTitlePage->PaintPage->Canvas->
                Pixels[textleft+YTempPage->PaintPage->Height-j-1][texttop + i] =
                YTempPage->PaintPage->Canvas->Pixels[i][j];
      UpdatePaintPage(YTitlePage);
   }
}
//---------------------------------------------------------------------------
inline void TXYPlot::DrawPlotsAndLabels(void)
{
   DrawAllPlots();
   DrawXAxisLabels();
   DrawYAxisLabels();
}
//---------------------------------------------------------------------------
void __fastcall TXYPlot::Paint(void)
{
// Event handler for repainting.
// Draws the Axis, plot labels, and plots.  Mouse Coordinates are drawn in the
// mouse move event handler
   if((OldWidth != Width)||(OldHeight!=Height)) // Size of the plot has changed
   {
      OldWidth = Width;
      OldHeight = Height;
      ResizePaintPage();
   }
   Redraw();
   TPaintBox::Paint();
}
//---------------------------------------------------------------------------
void TXYPlot::DrawMouseCoord(void)
{  // draws the mouse coordinates
   int texttop;
   AnsiString St;

   if ((FShowCoordinates)&&(Visible))                   // draw the coordinates
   {
      MouseCoordPage->PaintPage->Canvas->Font->Assign(FMouseCoordFont);
      MouseCoordPage->ClearPaintPage(Color);
      St = "x = " + St.FloatToStrF(FMouseX,FXLabelFormat,FXLabelPrecision,FXLabelDigits);
      MouseCoordPage->PaintPage->Canvas->TextOut(0,0,St);
      St = "y = " + St.FloatToStrF(FMouseY,FYLabelFormat,FYLabelPrecision,FYLabelDigits);
      texttop = MouseCoordPage->PaintPage->Canvas->TextHeight(St);
      MouseCoordPage->PaintPage->Canvas->TextOut(0,texttop + 3,St);
      UpdatePaintPage(MouseCoordPage);
      }
}
//---------------------------------------------------------------------------
void __fastcall TXYPlot::MouseMove(Classes::TShiftState Shift, int X, int Y)
//  Event handler for Mouse Move.  If the Mouse button was pressed before the
// start of the move, the plot is dragged.  If the Alt key and Mouse button were
// pressed, the plot is zoomed in and out.
{   TRect R;
   double DeltaX, DeltaY, DeltaFX, DeltaFY, xmin, xmax, ymin, ymax;
   long double xval, yval;

   TControl::MouseMove(Shift, X, Y);

   DeltaX = X - MX;
   DeltaY = Y - MY;
   MX = X;
   MY = Y;
   if (FXAxisFormat == Linear)
      DeltaFX = (FXMax-FXMin)*double(DeltaX)/PlotPage->PaintPage->Width;
   else if (FXAxisFormat == Log)
      DeltaFX = (XLogMax-XLogMin)*double(DeltaX)/PlotPage->PaintPage->Width;
   if (FYAxisFormat == Linear)
      DeltaFY = (FYMax-FYMin)*double(DeltaY)/PlotPage->PaintPage->Height;
   else if (FYAxisFormat == Log)
      DeltaFY = (YLogMax-YLogMin)*double(DeltaY)/PlotPage->PaintPage->Height;
   if(MouseControl)
   {
      if(DragPlot)                                     // mouse button was pressed
      {
         if(FYAxisFormat == Linear)
         {
            ymin = FYMin + DeltaFY;
            ymax = FYMax + DeltaFY;
         }
         else if (FYAxisFormat == Log)
         {
            ymin = FYMin * pow(10,DeltaFY);
            ymax = FYMax * pow(10,DeltaFY);
         }
         if(FXAxisFormat == Linear)
         {
            xmin = FXMin - DeltaFX;
            xmax = FXMax - DeltaFX;
         }
         else if (FXAxisFormat == Log)
         {
            xmin = FXMin / pow(10,DeltaFX);
            xmax = FXMax / pow(10,DeltaFX);
         }
         SetXYMinMax(xmin, xmax, ymin,ymax);
         if (!(FAutoRedraw))         //SetXYMaxMin didn't repaint so, do it here
         {
            DrawAllPlots();
            DrawXAxisLabels();
            DrawYAxisLabels();
         }
      }
      if(ZoomPlot)                     // mouse and alt were pressed
      {
         if(FYAxisFormat == Linear)
         {
            ymin = FYMin - DeltaFY;
            ymax = FYMax + DeltaFY;
         }
         else if (FYAxisFormat == Log)
         {
            ymin = FYMin / pow(10,DeltaFY);
            ymax = FYMax * pow(10,DeltaFY);
         }
         if(FXAxisFormat == Linear)
         {
            xmin = FXMin + DeltaFX;
            xmax = FXMax - DeltaFX;
         }
         else if (FXAxisFormat == Log)
         {
            xmin = FXMin * pow(10,DeltaFX);
            xmax = FXMax / pow(10,DeltaFX);
         }
         SetXYMinMax(xmin, xmax, ymin,ymax);
         if (!(FAutoRedraw))                              //SetXYMaxMin didn't repaint so, do it here
         {
            DrawAllPlots();
            DrawXAxisLabels();
            DrawYAxisLabels();
         }
      }
      if(BoxZoomPlot)
      {
         Canvas->Brush->Style = bsClear;
         Canvas->Pen->Color = clBlack;
         Canvas->Pen->Style = psDot;
         Canvas->Pen->Mode = pmNot;
         Canvas->Rectangle(ZoomBox->Left, ZoomBox->Top,
                           ZoomBox->Right, ZoomBox->Bottom);
         ZoomBox->Left = (X > ZoomBoxX) ? ZoomBoxX : X ;
         ZoomBox->Top = (Y > ZoomBoxY) ? ZoomBoxY : Y ;
         ZoomBox->Right = ZoomBox->Left + abs(X - ZoomBoxX);
         ZoomBox->Bottom = ZoomBox->Top + abs(Y - ZoomBoxY);
         Canvas->Rectangle(ZoomBox->Left, ZoomBox->Top,
                           ZoomBox->Right, ZoomBox->Bottom);
      }
   }
   FMouseX = PlotX(X);
   FMouseY = PlotY(Y);
   if((!(ZoomPlot))&&(!(DragPlot)))
      DrawMouseCoord();
}
//---------------------------------------------------------------------------
void __fastcall TXYPlot::MouseDown(TMouseButton Button,TShiftState Shift,
                        int X, int Y)
// Event handler for mouse down. If just the mouse button is pressed, DragPlot is
// set.  If both alt and mouse button, ZoomPlot is set.
{
   TShiftState LAltState, RAltState, LCtrlState;

   LAltState << ssAlt << ssLeft;
   RAltState << ssAlt << ssRight;
   LCtrlState << ssCtrl << ssLeft;
   TControl::MouseDown(Button, Shift, X, Y);
   if (Button == mbLeft)
   {
      if ((Shift == LAltState)||(Shift == RAltState))
         ZoomPlot = true;
      else if (Shift == LCtrlState)
         BoxZoomPlot = true;
      else
         DragPlot = true;
      }
   if ((BoxZoomPlot)&&(FMouseControl))
   {
      ZoomBox = new TRect;
      ZoomBox->Left  = X;
      ZoomBox->Top = Y;
      ZoomBox->Right = X+2;
      ZoomBox->Bottom = Y+2;
      ZoomBoxX = X;
      ZoomBoxY = Y;
      Canvas->Brush->Style = bsClear;
      Canvas->Pen->Color = clBlack;
      Canvas->Pen->Style = psDot;
      Canvas->Pen->Mode = pmNot;
      Canvas->Rectangle(ZoomBox->Left, ZoomBox->Top,
                        ZoomBox->Right, ZoomBox->Bottom);
   }
   MX = X;
   MY = Y;
}
//---------------------------------------------------------------------------
void __fastcall TXYPlot::MouseUp(TMouseButton Button,TShiftState Shift,
                         int X, int Y)
// Event handler for mouse up.  DragPlot and ZoomPlot are cleared.
{
   double xmax, xmin, ymax, ymin;
   TControl::MouseUp(Button, Shift, X, Y);
   DragPlot = false;
   ZoomPlot = false;
   if ((BoxZoomPlot)&&(FMouseControl))
   {
      Canvas->Brush->Style = bsClear;
      Canvas->Pen->Color = clBlack;
      Canvas->Pen->Style = psDot;
      Canvas->Pen->Mode = pmNot;
      Canvas->Rectangle(ZoomBox->Left, ZoomBox->Top,
                        ZoomBox->Right, ZoomBox->Bottom);
      xmin = PlotX(ZoomBox->Left);
      xmax = PlotX(ZoomBox->Right);
      ymax = PlotY(ZoomBox->Top);
      ymin = PlotY(ZoomBox->Bottom);
      SetXYMinMax(xmin, xmax, ymin, ymax);
      delete ZoomBox;
   }
   BoxZoomPlot = false;
}
//---------------------------------------------------------------------------
void __fastcall TXYPlot::DblClick(void)
// if autocales are set, double clicking autoscales the axis
{
   TControl::DblClick();
   if (MouseControl)
   {
      FindYMaxMin();
      FindXMaxMin();
      DrawPlotsAndLabels();
   }
}
//---------------------------------------------------------------------------
void __fastcall TXYPlot::SetAxisColor(TColor AColor)
// set AxisColor property
{
   FAxisColor = AColor;
   DrawAxis();
}
//---------------------------------------------------------------------------
void __fastcall TXYPlot::SetAxisStyle(TAxisStyle TS)
// set AxisStyle property
{
   FAxisStyle = TS;
   if(FAutoRedraw)
      DrawPlotsAndLabels();
}
//---------------------------------------------------------------------------
void __fastcall TXYPlot::SetXAxisFormat(TAxisFormat AF)
{  // set X axis to linear or log.
   FXAxisFormat = AF;
   FindXMaxMin();
   if(FXAxisFormat == Log)
   {
      if (FXMax <= 0) // all plots have x < 0
      {
         FXMax = 10;
         FXMin = 1;
      }
      else if (FXMin <=0)
      {
         FXMin = FXMax/10.0;
      }
      XLogMin = log10(FXMin);
      XLogMax = log10(FXMax);
   }
   if(FAutoRedraw)
      DrawPlotsAndLabels();
}
//---------------------------------------------------------------------------
void __fastcall TXYPlot::SetYAxisFormat(TAxisFormat AF)
{  // set Y axis to linear or log
   FYAxisFormat = AF;
   FindYMaxMin();
   if(FYAxisFormat == Log)
   {
      if (FYMax <= 0)
      {
         FYMax = 10;
         FYMin = 1;
      }
      else if (FYMin <=0)
      {
         FYMin = FYMax/10.0;
      }
      YLogMin = log10(FYMin);
      YLogMax = log10(FYMax);
   }
   if(FAutoRedraw)
      DrawPlotsAndLabels();
}
//---------------------------------------------------------------------------
void __fastcall TXYPlot::SetShowXLabel(bool XL)
// set ShowXLabel property
{
   FShowXLabel = XL;
   ResizePaintPage();
   if(FAutoRedraw)
      Redraw();
}
//---------------------------------------------------------------------------
void __fastcall TXYPlot::SetShowYLabel(bool YL)
// set ShowYLabel property
{
   FShowYLabel = YL;
   ResizePaintPage();
   if(FAutoRedraw)
      Redraw();
}
//---------------------------------------------------------------------------
void __fastcall TXYPlot::SetYLabelFormat(AnsiString::TStringFloatFormat LF)
{
   FYLabelFormat = LF;
   SetShowYLabel(FShowYLabel);
}
//---------------------------------------------------------------------------
void __fastcall TXYPlot::SetXLabelFormat(AnsiString::TStringFloatFormat LF)
{
   FXLabelFormat = LF;
   SetShowXLabel(FShowYLabel);
}
//---------------------------------------------------------------------------
void __fastcall TXYPlot::SetYLabelPrecision(int LP)
{
   FYLabelPrecision = LP;
   SetShowYLabel(FShowYLabel);
}
//---------------------------------------------------------------------------
void __fastcall TXYPlot::SetXLabelPrecision(int LP)
{
   FXLabelPrecision = LP;
   SetShowXLabel(FShowYLabel);
}
//---------------------------------------------------------------------------
void __fastcall TXYPlot::SetYLabelDigits(int LD)
{
   FYLabelDigits = LD;
   SetShowYLabel(FShowYLabel);
}
//---------------------------------------------------------------------------
void __fastcall TXYPlot::SetXLabelDigits(int LD)
{
   FXLabelDigits = LD;
   SetShowXLabel(FShowYLabel);
}
//---------------------------------------------------------------------------
void __fastcall TXYPlot::SetShowCoordinates(bool CO)
//  SetCoordinatesOn property, and clears the area where coordinates are displayed
{
   FShowCoordinates = CO;

   if(Visible)
   {
      if(FAutoRedraw)
      {
         MouseCoordPage->PaintPage->Canvas->Brush->Color = Color;
         MouseCoordPage->PaintPage->Canvas->FillRect(*(MouseCoordPage->PageRect));
         Canvas->CopyRect(*(MouseCoordPage->PositionRect),
                       MouseCoordPage->PaintPage->Canvas,*(MouseCoordPage->PageRect));
         DrawAllPlots();
      }
   }
}
//---------------------------------------------------------------------------
void __fastcall TXYPlot::SetYAutoScale(bool Yb)
// Sets the YAutoScale property.  If YAutoScale is set, the plot can still be dragged
// and zoomed with the mouse button
{
   int i,j;

   FYAutoScale = Yb;
   FindYMaxMin();
   if(FAutoRedraw)
      DrawPlotsAndLabels();
}
//---------------------------------------------------------------------------
void __fastcall TXYPlot::SetXAutoScale(bool Xb)
//  Sets the XAutoScale property.
{
   FXAutoScale = Xb;
   FindXMaxMin();
   if(FAutoRedraw)
      DrawPlotsAndLabels();
}
//---------------------------------------------------------------------------
void TXYPlot::CalculateLogMaxMin(void)
{  // calculates the log of XMax, XMin, YMax, and YMin
   if(FXAxisFormat == Log)
   {
      if (FXMax <= 0)
         FXMax = 10;
      if (FXMin <= 0)
         FXMin = FXMax/100;
      XLogMax = log10(FXMax);
      XLogMin = log10(FXMin);
   }
   if(FYAxisFormat == Log)
   {
      if (FYMax <= 0)
         FYMax = 10;
      if (FYMin <= 0)
         FYMin = FYMax/100;
      YLogMax = log10(FYMax);
      YLogMin = log10(FYMin);
   }
}
//---------------------------------------------------------------------------
void TXYPlot::CompareMaxMin(void)
{  // checks to make sure FYMax != to FYMin and FXMax != FXMin
   if (FYMax==FYMin)
   {
      if (FYAxisFormat == Linear)
          FYMax +=1;
      else
         FYMax = FYMin*10;
   }
   if(FXMin==FXMax)
   {
      if (FXAxisFormat == Linear)
          FXMax +=1;
      else
         FXMax = FXMin*10;
   }
}
//---------------------------------------------------------------------------
void TXYPlot::SetXYMinMax(double xmin, double xmax, double ymin, double ymax)
{  // method for setting all the axis ranges at once
   FYMax = ymax;
   FYMin = ymin;
   FXMax = xmax;
   FXMin = xmin;
   CalculateLogMaxMin();
   CompareMaxMin();
   if(FAutoRedraw)
      DrawPlotsAndLabels();
}
//---------------------------------------------------------------------------
void __fastcall TXYPlot::SetYMax(double YM)
// for manual scaling of the y axis
{
   FYMax = YM;
   CompareMaxMin();
   CalculateLogMaxMin();
   if(FAutoRedraw)
      DrawPlotsAndLabels();
}
//---------------------------------------------------------------------------
void __fastcall TXYPlot::SetYMin(double YM)
// for manual scaling of the y axis
{
   FYMin = YM;
   CompareMaxMin();
   CalculateLogMaxMin();
   if(FAutoRedraw)
      DrawPlotsAndLabels();
}
//---------------------------------------------------------------------------
void __fastcall TXYPlot::SetXMax(double XM)
// for manual scaling of the x axis
{
   FXMax = XM;
   CompareMaxMin();
   CalculateLogMaxMin();
   if(FAutoRedraw)
      DrawPlotsAndLabels();
}
//---------------------------------------------------------------------------
void __fastcall TXYPlot::SetXMin(double XM)
// for manual scaling of the x axis
{
   FXMin = XM;
   CompareMaxMin();
   CalculateLogMaxMin();
   if(FAutoRedraw)
      DrawPlotsAndLabels();
}
//---------------------------------------------------------------------------
void __fastcall TXYPlot::SetShowHorzGrid(bool SG)
{
   FShowHorzGrid = SG;
   if(FAutoRedraw)
      DrawAllPlots();
}
//---------------------------------------------------------------------------
void __fastcall TXYPlot::SetShowVertGrid(bool SG)
{
   FShowVertGrid = SG;
   if(FAutoRedraw)
      DrawAllPlots();
}
//---------------------------------------------------------------------------
void __fastcall TXYPlot::SetShowHorzTicks(bool ST)
{
   FShowHorzTicks = ST;
   if(FAutoRedraw)
      DrawAllPlots();
}
//---------------------------------------------------------------------------
void __fastcall TXYPlot::SetShowVertTicks(bool ST)
{
   FShowVertTicks = ST;
   if(FAutoRedraw)
      DrawAllPlots();
}
//---------------------------------------------------------------------------
void __fastcall TXYPlot::SetVertTickLength(int TL)
{
   FVertTickLength = TL;
   if (FVertTickLength < 0) FVertTickLength = 0;
   if (FVertTickLength > Height) FVertTickLength = Height;
   if(FAutoRedraw)
      DrawAllPlots();
}
//---------------------------------------------------------------------------
void __fastcall TXYPlot::SetHorzTickLength(int TL)
{
   FHorzTickLength = TL;
   if (FHorzTickLength < 0) FHorzTickLength = 0;
   if (FHorzTickLength > Width) FHorzTickLength = Height;
   if(FAutoRedraw)
      DrawAllPlots();
}
//---------------------------------------------------------------------------
void __fastcall TXYPlot::SetNumHorzGrid(int NG)
{
   if (NG>=0)
      FNumHorzGrid = NG;
   else
      FNumHorzGrid = 0;
   if(FAutoRedraw)
      DrawPlotsAndLabels();
}
//---------------------------------------------------------------------------
void __fastcall TXYPlot::SetNumVertGrid(int NG)
{
   if(NG>=0)
      FNumVertGrid = NG;
   else
      FNumVertGrid = 0;
   if(FAutoRedraw)
      DrawPlotsAndLabels();
}
//---------------------------------------------------------------------------
void __fastcall TXYPlot::SetGridColor(TColor GC)
{
   FGridColor = GC;
   if(FAutoRedraw)
      DrawAllPlots();
}
//---------------------------------------------------------------------------
void __fastcall TXYPlot::SetPointSize(int PS)
{
   if (PS<1)
      FPointSize = 1;
   else if (PS>10)
      FPointSize = 10;
   else
      FPointSize = PS;
   if(FAutoRedraw)
      DrawAllPlots();
}
//---------------------------------------------------------------------------
void __fastcall TXYPlot::SetAutoRedraw(bool AR)
{
   FAutoRedraw = AR;
   if(FAutoRedraw)
      Redraw();
}
//---------------------------------------------------------------------------
void __fastcall TXYPlot::SetShowXTitle(bool ST)
{
   FShowXTitle = ST;
   ResizePaintPage();
   if(FAutoRedraw)
      Redraw();
}
//---------------------------------------------------------------------------
void __fastcall TXYPlot::SetShowYTitle(bool ST)
{
   FShowYTitle = ST;
   ResizePaintPage();
   if(FAutoRedraw)
      Redraw();
}
//---------------------------------------------------------------------------
void __fastcall TXYPlot::SetShowPlotTitle(bool ST)
{
   FShowPlotTitle = ST;
   ResizePaintPage();
   if(FAutoRedraw)
      Redraw();
}
//---------------------------------------------------------------------------
void __fastcall TXYPlot::SetXTitle(AnsiString XT)
{
   FXTitle = XT;
   if (FAutoRedraw)
      DrawTitles();
}
//---------------------------------------------------------------------------
void __fastcall TXYPlot::SetYTitle(AnsiString YT)
{
   FYTitle = YT;
   ResizePaintPage();
   if (FAutoRedraw)
      DrawTitles();
}
//---------------------------------------------------------------------------
void __fastcall TXYPlot::SetPlotTitle(AnsiString PT)
{
   FPlotTitle = PT;
   if (FAutoRedraw)
      DrawTitles();
}
//---------------------------------------------------------------------------
void __fastcall TXYPlot::SetPlotTitleFont(TFont* TF)
{
   FPlotTitleFont->Assign(TF);
   PlotTitlePage->PaintPage->Canvas->Font->Assign(FPlotTitleFont);
   ResizePaintPage();
   if(FAutoRedraw)
      Redraw();
}
//---------------------------------------------------------------------------
void __fastcall TXYPlot::SetAxisTitleFont(TFont* TF)
{
   FAxisTitleFont->Assign(TF);
   XTitlePage->PaintPage->Canvas->Font->Assign(FAxisTitleFont);
   YTitlePage->PaintPage->Canvas->Font->Assign(FAxisTitleFont);
   ResizePaintPage();
   if(FAutoRedraw)
      Redraw();
}
//---------------------------------------------------------------------------
void __fastcall TXYPlot::SetAxisLabelFont(TFont* LF)
{
   FAxisLabelFont->Assign(LF);
   XLabelPage->PaintPage->Canvas->Font->Assign(FAxisLabelFont);
   YLabelPage->PaintPage->Canvas->Font->Assign(FAxisLabelFont);
   ResizePaintPage();
   if(FAutoRedraw)
      Redraw();
}
//---------------------------------------------------------------------------
void __fastcall TXYPlot::SetMouseCoordFont(TFont* CF)
{
   FMouseCoordFont->Assign(CF);
   MouseCoordPage->PaintPage->Canvas->Font->Assign(FMouseCoordFont);
   ResizePaintPage();
   if(FAutoRedraw)
      DrawPlotsAndLabels();
}
//---------------------------------------------------------------------------
bool TXYPlot::SetPlotColor(TColor PColor, int PNumber)
// sets the color of plot number PNumber. If the plot exists, a 1 is returned.
// If a plot with index PNumber is not found, a zero is returned.
{
   TPlotListMember* CurrentPtr;
   bool PlotFound;

   PlotFound = PlotList->FindPlot(CurrentPtr, PNumber);
   if(PlotFound)
   {
      CurrentPtr->Color = PColor;
      DrawOnePlot(CurrentPtr);
      RedrawScreen();
   }
   return (PlotFound);
}
//---------------------------------------------------------------------------
bool TXYPlot::GetPlotStyle(TPlotStyle& PStyle, int PNumber)
{
   TPlotListMember* CurrentPtr;
   bool PlotFound;

   PlotFound = PlotList->FindPlot(CurrentPtr, PNumber);
   if(PlotFound)
      PStyle = CurrentPtr->PlotStyle;

   return(PlotFound);
}
//---------------------------------------------------------------------------
bool TXYPlot::GetPlotColor(TColor& PColor, int PNumber)
{
   TPlotListMember* CurrentPtr;
   bool PlotFound;

   PlotFound = PlotList->FindPlot(CurrentPtr, PNumber);
   if(PlotFound)
      PColor = CurrentPtr->Color ;

   return(PlotFound);
}
//---------------------------------------------------------------------------
bool TXYPlot::GetPlotNumPoints(int& NP, int PNumber)
{
   TPlotListMember* CurrentPtr;
   bool PlotFound;

   PlotFound = PlotList->FindPlot(CurrentPtr, PNumber);
   if(PlotFound)
      NP = CurrentPtr->NumPoints ;

   return(PlotFound);
}
//---------------------------------------------------------------------------
bool TXYPlot::GetPlotX(double x[], int PNumber)
{
   int i;
   TPlotListMember* CurrentPtr;
   bool PlotFound;

   PlotFound = PlotList->FindPlot(CurrentPtr, PNumber);
   if(PlotFound)
      for(i=0;i<CurrentPtr->NumPoints;i++)
         x[i] = CurrentPtr->x[i];
   return(PlotFound);
}
//---------------------------------------------------------------------------
bool TXYPlot::GetPlotY(double y[], int PNumber)
{
   int i;
   TPlotListMember* CurrentPtr;
   bool PlotFound;

   PlotFound = PlotList->FindPlot(CurrentPtr, PNumber);
   if(PlotFound)
      for(i=0;i<CurrentPtr->NumPoints;i++)
         y[i] = CurrentPtr->y[i];
   return(PlotFound);
}
//---------------------------------------------------------------------------
bool TXYPlot::SetPlotStyle(TPlotStyle PStyle, int PNumber)
// sets the style of plot number PNumber. If the plot exists, a 1 is returned.
// If a plot with index PNumber is not found, a zero is returned.
{
   TPlotListMember* CurrentPtr;
   TColor TmpColor;
   bool PlotFound;

   PlotFound = PlotList->FindPlot(CurrentPtr, PNumber);
   if(PlotFound)
   {
      CurrentPtr->PlotStyle = PStyle;
      if(FAutoRedraw)
      DrawAllPlots();
   }

   return (PlotFound);
}
//---------------------------------------------------------------------------
void TXYPlot::FindYMaxMin(void)
// if y autoscaling is on, finds the max and min values of the y axis
{
   int i;
   TPlotListMember* CurrentPtr;

   if(FYAutoScale)
   {
      CurrentPtr = PlotList->FirstPtr;
      if(CurrentPtr != NULL)
      {
         FYMax = CurrentPtr->y[0];
         FYMin = CurrentPtr->y[0];
      }
      else
      {
         FYMax = 1;
         FYMin = -1;
      }
      while(CurrentPtr !=NULL)
      {
         for(i=0;i<CurrentPtr->NumPoints;i++)
         {
            if (FYMax < CurrentPtr->y[i])
               FYMax = CurrentPtr->y[i];
            if (FYMin > CurrentPtr->y[i])
               FYMin = CurrentPtr->y[i];
         }
         CurrentPtr = CurrentPtr->NextPtr;
      }
      CompareMaxMin();
      CalculateLogMaxMin();
   }
}
//---------------------------------------------------------------------------
void TXYPlot::FindXMaxMin(void)
// if autoscaling is on, finds the max and min values of the x axis.
{
   int i;
   TPlotListMember* CurrentPtr;

   if(FXAutoScale)
   {
      CurrentPtr = PlotList->FirstPtr;
      if(CurrentPtr != NULL)              // no plots have been stored
      {
         FXMax = CurrentPtr->x[0];
         FXMin = CurrentPtr->x[0];
      }
      else
      {
         FXMax = 1;
         FXMin = -1;
      }
      while(CurrentPtr !=NULL)
      {
         for(i=0;i<CurrentPtr->NumPoints;i++)
         {
            if (FXMax < CurrentPtr->x[i])
               FXMax = CurrentPtr->x[i];
            if (FXMin > CurrentPtr->x[i])
               FXMin = CurrentPtr->x[i];
         }
         CurrentPtr = CurrentPtr->NextPtr;
      }
      CompareMaxMin();
      CalculateLogMaxMin();
   }
}
//------------------------------------------------------------------------
void __fastcall TXYPlot::Change()
{
   if(FOnChange)
   {
      FOnChange(this);
   }
}
//---------------------------------------------------------------------------


